@@ -127,17 +127,6 @@ static GQuark quark_object_animation = 0;
 
 G_DEFINE_TYPE (ClutterAnimation, clutter_animation, G_TYPE_OBJECT);
 
-static void
-on_actor_dispose (gpointer  user_data,
-                  GObject  *actor_pointer)
-{
-  ClutterAnimation *self = user_data;
-
-  if (self->priv->object == actor_pointer)
-    g_object_unref (self);
-}
-
-
 static void
 clutter_animation_real_completed (ClutterAnimation *self)
 {
@@ -179,7 +168,6 @@ clutter_animation_real_completed (ClutterAnimation *self)
   if (animation == self)
     {
       g_object_set_qdata (priv->object, quark_object_animation, NULL);
-      g_object_weak_unref (priv->object, on_actor_dispose, self);
       g_object_unref (animation);
     }
 }
@@ -778,6 +766,59 @@ clutter_animation_update_interval (ClutterAnimation *animation,
   clutter_animation_update_property_internal (animation, pspec, interval);
 }
 
+/**
+ * clutter_animation_update:
+ * @animation: a #ClutterAnimation
+ * @property_name: name of the property
+ * @final: The final value of the property
+ *
+ * Updates the @final value of the interval for @property_name
+ *
+ * Return value: (transfer none): The animation itself.
+ *
+ * Since: 1.0
+ */
+ClutterAnimation *
+clutter_animation_update (ClutterAnimation *animation,
+                          const gchar      *property_name,
+                          const GValue     *final)
+{
+  ClutterAnimationPrivate *priv;
+  ClutterInterval *interval;
+
+  g_return_val_if_fail (CLUTTER_IS_ANIMATION (animation), NULL);
+  g_return_val_if_fail (property_name != NULL, NULL);
+  g_return_val_if_fail (final != NULL, NULL);
+  g_return_val_if_fail (G_VALUE_TYPE (final) != G_TYPE_INVALID, NULL);
+
+  priv = animation->priv;
+
+  interval = clutter_animation_get_interval (animation, property_name);
+  if (interval == NULL)
+    {
+      g_warning ("Cannot update property '%s': the animation has "
+                 "no bound property with that name",
+                 property_name);
+      return NULL;
+    }
+
+  if (!g_value_type_compatible (G_VALUE_TYPE (final),
+                                clutter_interval_get_value_type (interval)))
+    {
+      g_warning ("Cannot update property '%s': the interval value of "
+                 "type '%s' is not compatible with the property value "
+                 "of type '%s'",
+                 property_name,
+                 g_type_name (clutter_interval_get_value_type (interval)),
+                 g_type_name (G_VALUE_TYPE (final)));
+      return NULL;
+    }
+
+  clutter_interval_set_final_value (interval, final);
+
+  return animation;
+}
+
 /**
  * clutter_animation_get_interval:
  * @animation: a #ClutterAnimation
@@ -1690,7 +1731,6 @@ animation_create_for_actor (ClutterActor *actor)
       animation = clutter_animation_new ();
       clutter_animation_set_object (animation, object);
       g_object_set_qdata (object, quark_object_animation, animation);
-      g_object_weak_ref (object, on_actor_dispose, animation);
 
       CLUTTER_NOTE (ANIMATION,
                     "Created new Animation [%p] for actor [%p]",
