@@ -28,62 +28,6 @@
  *
  * Functions to retrieve various global Clutter resources and other utility
  * functions for mainloops, events and threads
- *
- * <refsect2 id="clutter-Threading-Model">
- *   <title>Threading Model</title>
- *   <para>Clutter is <emphasis>thread-aware</emphasis>: all operations
- *   performed by Clutter are assumed to be under the big Clutter lock,
- *   which is created when the threading is initialized through
- *   clutter_threads_init().</para>
- *   <example id="example-Thread-Init">
- *     <title>Thread Initialization</title>
- *     <para>The code below shows how to correctly initialize Clutter
- *     in a multi-threaded environment. These operations are mandatory for
- *     applications that wish to use threads with Clutter.</para>
- *     <programlisting>
- * int
- * main (int argc, char *argv[])
- * {
- *   /&ast; initialize GLib's threading support &ast;/
- *   g_thread_init (NULL);
- *
- *   /&ast; initialize Clutter's threading support &ast;/
- *   clutter_threads_init ();
- *
- *   /&ast; initialize Clutter &ast;/
- *   clutter_init (&amp;argc, &amp;argv);
- *
- *   /&ast; program code &ast;/
- *
- *   /&ast; acquire the main lock &ast;/
- *   clutter_threads_enter ();
- *
- *   /&ast; start the main loop &ast;/
- *   clutter_main ();
- *
- *   /&ast; release the main lock &ast;/
- *   clutter_threads_leave ();
- *
- *   /&ast; clean up &ast;/
- *   return 0;
- * }
- *     </programlisting>
- *   </example>
- *   <para>This threading model has the caveat that it is only safe to call
- *   Clutter's API when the lock has been acquired &mdash; which happens
- *   between pairs of clutter_threads_enter() and clutter_threads_leave()
- *   calls.</para>
- *   <para>The only safe and portable way to use the Clutter API in a
- *   multi-threaded environment is to never access the API from a thread that
- *   did not call clutter_init() and clutter_main().</para>
- *   <para>The common pattern for using threads with Clutter is to use worker
- *   threads to perform blocking operations and then install idle or timeour
- *   sources with the result when the thread finished.</para>
- *   <para>Clutter provides thread-aware variants of g_idle_add() and
- *   g_timeout_add() that acquire the Clutter lock before invoking the provided
- *   callback: clutter_threads_add_idle() and
- *   clutter_threads_add_timeout().</para>
- * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -94,6 +38,10 @@
 #include <glib/gi18n-lib.h>
 #include <locale.h>
 
+#ifdef USE_GDKPIXBUF
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#endif
+
 #include "clutter-event.h"
 #include "clutter-backend.h"
 #include "clutter-main.h"
@@ -151,7 +99,8 @@ static const GDebugKey clutter_debug_keys[] = {
   { "multistage", CLUTTER_DEBUG_MULTISTAGE },
   { "animation", CLUTTER_DEBUG_ANIMATION },
   { "layout", CLUTTER_DEBUG_LAYOUT },
-  { "nop-picking", CLUTTER_DEBUG_NOP_PICKING }
+  { "nop-picking", CLUTTER_DEBUG_NOP_PICKING },
+  { "dump-pick-buffers", CLUTTER_DEBUG_DUMP_PICK_BUFFERS }
 };
 #endif /* CLUTTER_ENABLE_DEBUG */
 
@@ -392,6 +341,18 @@ _clutter_id_to_color (guint id, ClutterColor *col)
   col->green = green;
   col->blue  = blue;
   col->alpha = 0xff;
+
+  /* XXX: We rotate the nibbles of the colors here so that there is a
+   * visible variation between colors of sequential actor identifiers;
+   * otherwise pick buffers dumped to an image will pretty much just look
+   * black.
+   */
+  if (G_UNLIKELY (clutter_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS))
+    {
+      col->red = (col->red << 4) | (col->red >> 4);
+      col->green = (col->green << 4) | (col->green >> 4);
+      col->blue = (col->blue << 4) | (col->blue >> 4);
+    }
 }
 
 guint
@@ -406,9 +367,28 @@ _clutter_pixel_to_id (guchar pixel[4])
   /* reduce the pixel components to the number of bits actually used of the
    * 8bits.
    */
-  red   = pixel[0] >> (8 - ctx->fb_r_mask);
-  green = pixel[1] >> (8 - ctx->fb_g_mask);
-  blue  = pixel[2] >> (8 - ctx->fb_b_mask);
+  if (G_UNLIKELY (clutter_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS))
+    {
+      guchar tmp;
+
+      /* XXX: In _clutter_id_to_color we rotated the nibbles of the colors so
+       * that there is a visible variation between colors of sequential actor
+       * identifiers (otherwise pick buffers dumped to an image will pretty
+       * much just look black.) Here we reverse that rotation.
+       */
+      tmp = ((pixel[0] << 4) | (pixel[0] >> 4));
+      red = tmp >> (8 - ctx->fb_r_mask);
+      tmp = ((pixel[1] << 4) | (pixel[1] >> 4));
+      green = tmp >> (8 - ctx->fb_g_mask);
+      tmp = ((pixel[2] << 4) | (pixel[2] >> 4));
+      blue = tmp >> (8 - ctx->fb_b_mask);
+    }
+  else
+    {
+      red   = pixel[0] >> (8 - ctx->fb_r_mask);
+      green = pixel[1] >> (8 - ctx->fb_g_mask);
+      blue  = pixel[2] >> (8 - ctx->fb_b_mask);
+    }
 
   /* divide potentially by two if 'fuzzy' */
   red   = red   >> (ctx->fb_r_mask - ctx->fb_r_mask_used);
@@ -422,6 +402,70 @@ _clutter_pixel_to_id (guchar pixel[4])
   return id;
 }
 
+#ifdef USE_GDKPIXBUF
+static void
+pixbuf_free (guchar *pixels, gpointer data)
+{
+  g_free (pixels);
+}
+#endif
+
+static void
+read_pixels_to_file (char *filename_stem,
+                     int x,
+                     int y,
+                     int width,
+                     int height)
+{
+#ifdef USE_GDKPIXBUF
+  GLubyte *data;
+  GdkPixbuf *pixbuf;
+  static int read_count = 0;
+  GdkPixbuf *flipped;
+
+  data = g_malloc (4 * width * height);
+  glReadPixels (0, 0, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data);
+  pixbuf = gdk_pixbuf_new_from_data (data,
+                                     GDK_COLORSPACE_RGB,
+                                     TRUE, /* has alpha */
+                                     8, /* bits per sample */
+                                     width, /* width */
+                                     height, /* height */
+                                     width * 4, /* rowstride */
+                                     pixbuf_free, /* callback to free data */
+                                     NULL); /* callback data */
+  if (pixbuf)
+    {
+      flipped = gdk_pixbuf_flip (pixbuf, FALSE);
+      g_object_unref (pixbuf);
+    }
+
+  if (flipped)
+    {
+      char *filename =
+        g_strdup_printf ("%s-%05d.png", filename_stem, read_count);
+      GError *error = NULL;
+      if (!gdk_pixbuf_save (flipped, filename, "png", &error, NULL))
+        {
+          g_warning ("Failed to save pick buffer to file %s: %s",
+                     filename, error->message);
+          g_error_free (error);
+        }
+      g_free (filename);
+      g_object_unref (flipped);
+      read_count++;
+    }
+#else
+  static gboolean seen = FALSE;
+  if (!seen)
+    {
+      g_warning ("dumping buffers to an image isn't supported on platforms "
+                 "without gdk pixbuf support\n");
+      seen = TRUE;
+    }
+#endif
+}
+
 ClutterActor *
 _clutter_do_pick (ClutterStage   *stage,
 		  gint            x,
@@ -445,7 +489,9 @@ _clutter_do_pick (ClutterStage   *stage,
   /* needed for when a context switch happens */
   _clutter_stage_maybe_setup_viewport (stage);
 
-  cogl_clip_push_window_rect (x, y, 1, 1);
+  if (G_LIKELY (!(clutter_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS)))
+    cogl_clip_push_window_rect (x, y, 1, 1);
+
   cogl_color_set_from_4ub (&white, 255, 255, 255, 255);
   cogl_disable_fog ();
   cogl_clear (&white,
@@ -463,7 +509,9 @@ _clutter_do_pick (ClutterStage   *stage,
   context->pick_mode = mode;
   clutter_actor_paint (CLUTTER_ACTOR (stage));
   context->pick_mode = CLUTTER_PICK_NONE;
-  cogl_clip_pop ();
+
+  if (G_LIKELY (!(clutter_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS)))
+    cogl_clip_pop ();
 
   /* Calls should work under both GL and GLES, note GLES needs RGBA */
   glGetIntegerv(GL_VIEWPORT, viewport);
@@ -474,6 +522,9 @@ _clutter_do_pick (ClutterStage   *stage,
   /* Read the color of the screen co-ords pixel */
   glReadPixels (x, viewport[3] - y -1, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixel);
 
+  if (G_UNLIKELY (clutter_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS))
+    read_pixels_to_file ("pick-buffer", 0, 0, viewport[2], viewport[3]);
+
   /* Restore whether GL_DITHER was enabled */
   if (dither_was_on)
     glEnable (GL_DITHER);
@@ -1608,10 +1659,8 @@ clutter_init_with_args (int            *argc,
 
   if (!ctx->defer_display_setup)
     {
-#if 0
       if (argc && *argc > 0 && *argv)
 	g_set_prgname ((*argv)[0]);
-#endif
 
       context = g_option_context_new (parameter_string);
 
@@ -1716,10 +1765,8 @@ clutter_init (int    *argc,
 
   if (!ctx->defer_display_setup)
     {
-#if 0
       if (argc && *argc > 0 && *argv)
 	g_set_prgname ((*argv)[0]);
-#endif
 
       /* parse_args will trigger backend creation and things like
        * DISPLAY connection etc.
