@@ -87,18 +87,18 @@ static int drm_wait_vblank(int fd, drm_wait_vblank_t *vbl)
 {
     int ret, rc;
 
-    do 
+    do
       {
         ret = ioctl(fd, DRM_IOCTL_WAIT_VBLANK, vbl);
         vbl->request.type &= ~DRM_VBLANK_RELATIVE;
         rc = errno;
-      } 
+      }
     while (ret && rc == EINTR);
-    
+
     return rc;
 }
 
-#endif 
+#endif
 
 G_CONST_RETURN gchar*
 clutter_backend_glx_get_vblank_method (void)
@@ -131,8 +131,8 @@ clutter_backend_glx_post_parse (ClutterBackend  *backend,
 
   if (clutter_backend_x11_post_parse (backend, error))
     {
-      if (!glXQueryVersion (backend_x11->xdpy, &glx_major, &glx_minor) 
-          || !(glx_major > 1 || glx_minor > 1)) 
+      if (!glXQueryVersion (backend_x11->xdpy, &glx_major, &glx_minor)
+          || !(glx_major > 1 || glx_minor > 1))
         {
           g_set_error (error, CLUTTER_INIT_ERROR,
                        CLUTTER_INIT_ERROR_BACKEND,
@@ -149,8 +149,8 @@ clutter_backend_glx_post_parse (ClutterBackend  *backend,
 
 static const GOptionEntry entries[] =
 {
-  { "vblank", 0, 
-    0, 
+  { "vblank", 0,
+    0,
     G_OPTION_ARG_STRING, &clutter_vblank_name,
     N_("VBlank method to be used (none, dri or glx)"), "METHOD"
   },
@@ -212,7 +212,7 @@ clutter_backend_glx_constructor (GType                  gtype,
 
   g_warning ("Attempting to create a new backend object. This should "
              "never happen, so we return the singleton instance.");
-  
+
   return g_object_ref (backend_singleton);
 }
 
@@ -231,7 +231,7 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
   ClutterBackendGLX  *backend_glx = CLUTTER_BACKEND_GLX (backend);
   const gchar        *glx_extensions = NULL;
   ClutterFeatureFlags flags;
-  
+
   flags = clutter_backend_x11_get_features (backend);
   flags |= CLUTTER_FEATURE_STAGE_MULTIPLE;
 
@@ -251,7 +251,7 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
                 glGetString (GL_VERSION),
                 glGetString (GL_EXTENSIONS));
 
-  glx_extensions = 
+  glx_extensions =
     glXQueryExtensionsString (clutter_x11_get_default_display (),
 			      clutter_x11_get_default_screen ());
 
@@ -264,7 +264,7 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
     }
   else
     {
-      /* We try two GL vblank syncing mechanisms.  
+      /* We try two GL vblank syncing mechanisms.
        * glXSwapIntervalSGI is tried first, then glXGetVideoSyncSGI.
        *
        * glXSwapIntervalSGI is known to work with Mesa and in particular
@@ -276,10 +276,10 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
        * to be investigated. glXGetVideoSyncSGI on ATI at least seems to have
        * no effect.
       */
-      if (!check_vblank_env ("dri") && 
+      if (!check_vblank_env ("dri") &&
           cogl_check_extension ("GLX_SGI_swap_control", glx_extensions))
         {
-          backend_glx->swap_interval = 
+          backend_glx->swap_interval =
             (SwapIntervalProc) cogl_get_proc_address ("glXSwapIntervalSGI");
 
           CLUTTER_NOTE (BACKEND, "attempting glXSwapIntervalSGI vblank setup");
@@ -298,22 +298,22 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
             CLUTTER_NOTE (BACKEND, "glXSwapIntervalSGI vblank setup failed");
         }
 
-      if (!check_vblank_env ("dri") && 
+      if (!check_vblank_env ("dri") &&
           !(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK) &&
           cogl_check_extension ("GLX_SGI_video_sync", glx_extensions))
         {
           CLUTTER_NOTE (BACKEND, "attempting glXGetVideoSyncSGI vblank setup");
 
-          backend_glx->get_video_sync = 
+          backend_glx->get_video_sync =
             (GetVideoSyncProc) cogl_get_proc_address ("glXGetVideoSyncSGI");
 
-          backend_glx->wait_video_sync = 
+          backend_glx->wait_video_sync =
             (WaitVideoSyncProc) cogl_get_proc_address ("glXWaitVideoSyncSGI");
 
           if ((backend_glx->get_video_sync != NULL) &&
               (backend_glx->wait_video_sync != NULL))
             {
-              CLUTTER_NOTE (BACKEND, 
+              CLUTTER_NOTE (BACKEND,
                             "glXGetVideoSyncSGI vblank setup success");
 
               backend_glx->vblank_type = CLUTTER_VBLANK_GLX;
@@ -324,7 +324,7 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
             CLUTTER_NOTE (BACKEND, "glXGetVideoSyncSGI vblank setup failed");
         }
 #ifdef __linux__
-      /* 
+      /*
        * DRI is really an extreme fallback -rumoured to work with Via chipsets
       */
       if (!(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK))
@@ -354,9 +354,74 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
   return flags;
 }
 
+/* It seems the GLX spec never defined an invalid GLXFBConfig that
+ * we could overload as an indication of error, so we have to return
+ * an explicit boolean status. */
+gboolean
+_clutter_backend_glx_get_fbconfig (ClutterBackendGLX *backend_glx,
+                                   GLXFBConfig       *config)
+{
+  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend_glx);
+  int attributes[] = {
+    GLX_DRAWABLE_TYPE,  GLX_WINDOW_BIT,
+    GLX_RENDER_TYPE,    GLX_RGBA_BIT,
+    GLX_DOUBLEBUFFER,   GL_TRUE,
+    GLX_RED_SIZE,       1,
+    GLX_GREEN_SIZE,     1,
+    GLX_BLUE_SIZE,      1,
+    GLX_ALPHA_SIZE,     1,
+    GLX_DEPTH_SIZE,     1,
+    GLX_STENCIL_SIZE,   1,
+    None
+  };
+  GLXFBConfig *configs = NULL;
+  int n_configs;
+
+  if (backend_x11->xdpy == None || backend_x11->xscreen == None)
+    return FALSE;
+
+  if (backend_glx->found_fbconfig)
+    {
+      *config = backend_glx->fbconfig;
+      return TRUE;
+    }
+
+  CLUTTER_NOTE (BACKEND,
+                "Retrieving GL fbconfig, dpy: %p, xscreen; %p (%d)",
+                backend_x11->xdpy,
+                backend_x11->xscreen,
+                backend_x11->xscreen_num);
+
+  configs = glXChooseFBConfig (backend_x11->xdpy,
+                               backend_x11->xscreen_num,
+                               attributes,
+                               &n_configs);
+  if (configs)
+    {
+      *config = configs[0];
+      backend_glx->found_fbconfig = TRUE;
+      backend_glx->fbconfig = configs[0];
+      XFree (configs);
+      return TRUE;
+    }
+  else
+    return FALSE;
+}
+
+static XVisualInfo *
+clutter_backend_glx_get_visual_info (ClutterBackendX11 *backend_x11)
+{
+  ClutterBackendGLX *backend_glx = CLUTTER_BACKEND_GLX (backend_x11);
+  GLXFBConfig config;
+
+  if (!_clutter_backend_glx_get_fbconfig (backend_glx, &config))
+    return NULL;
+
+  return glXGetVisualFromFBConfig (backend_x11->xdpy, config);
+}
+
 static gboolean
 clutter_backend_glx_create_context (ClutterBackend  *backend,
-                                    gboolean         is_offscreen,
                                     GError         **error)
 {
   ClutterBackendGLX *backend_glx = CLUTTER_BACKEND_GLX (backend);
@@ -364,68 +429,72 @@ clutter_backend_glx_create_context (ClutterBackend  *backend,
 
   if (backend_glx->gl_context == None)
     {
-      XVisualInfo *xvisinfo;
+      GLXFBConfig config;
+      gboolean is_direct;
 
-      xvisinfo =
-        clutter_backend_x11_get_visual_info (backend_x11, is_offscreen);
+      if (!_clutter_backend_glx_get_fbconfig (backend_glx, &config))
+        {
+          g_set_error (error, CLUTTER_INIT_ERROR,
+                       CLUTTER_INIT_ERROR_BACKEND,
+                       "Unable to find suitable fbconfig for GL context");
+          return FALSE;
+        }
 
-      CLUTTER_NOTE (GL, "Creating GL Context (display: %p, %s)",
-                    backend_x11->xdpy,
-                    is_offscreen ? "offscreen" : "onscreen");
-
-      backend_glx->gl_context = glXCreateContext (backend_x11->xdpy,
-                                                  xvisinfo,
-                                                  0,
-                                                  is_offscreen ? False : True);
-
-      XFree (xvisinfo);
+      CLUTTER_NOTE (GL, "Creating GL Context (display: %p)",
+                    backend_x11->xdpy);
+      backend_glx->gl_context =
+        glXCreateNewContext (backend_x11->xdpy,
+                             config,
+                             GLX_RGBA_TYPE,
+                             NULL,
+                             True);
 
       if (backend_glx->gl_context == None)
         {
           g_set_error (error, CLUTTER_INIT_ERROR,
                        CLUTTER_INIT_ERROR_BACKEND,
-                       "Unable to create suitable %s GL context",
-                       is_offscreen ? "offscreen" : "onscreen");
+                       "Unable to create suitable GL context");
           return FALSE;
         }
 
-      if (!is_offscreen)
-        {
-          gboolean is_direct;
+      is_direct = glXIsDirect (backend_x11->xdpy,
+                               backend_glx->gl_context);
 
-          is_direct = glXIsDirect (backend_x11->xdpy,
-                                   backend_glx->gl_context);
-
-          CLUTTER_NOTE (GL, "Setting %s context",
-                        is_direct ? "direct" : "indirect");
-          _cogl_set_indirect_context (!is_direct);
-        }
+      CLUTTER_NOTE (GL, "Setting %s context",
+                    is_direct ? "direct" : "indirect");
+      _cogl_set_indirect_context (!is_direct);
     }
 
   return TRUE;
 }
 
 static void
-clutter_backend_glx_ensure_context (ClutterBackend *backend, 
+clutter_backend_glx_ensure_context (ClutterBackend *backend,
                                     ClutterStage   *stage)
 {
-  if (stage == NULL)
+  ClutterStageWindow *impl;
+
+  /* if there is no stage, the stage is being destroyed or it has no
+   * implementation attached to it then we clear the GL context
+   */
+  if (stage == NULL ||
+      (CLUTTER_PRIVATE_FLAGS (stage) & CLUTTER_ACTOR_IN_DESTRUCTION) ||
+      ((impl = _clutter_stage_get_window (stage)) == NULL))
     {
       ClutterBackendX11 *backend_x11;
 
       backend_x11 = CLUTTER_BACKEND_X11 (backend);
       CLUTTER_NOTE (MULTISTAGE, "Clearing all context");
 
-      glXMakeCurrent (backend_x11->xdpy, None, NULL);
+      glXMakeContextCurrent (backend_x11->xdpy, None, None, NULL);
     }
   else
     {
-      ClutterBackendGLX  *backend_glx;
-      ClutterStageGLX    *stage_glx;
-      ClutterStageX11    *stage_x11;
-      ClutterStageWindow *impl;
+      ClutterBackendGLX *backend_glx;
+      ClutterBackendX11 *backend_x11;
+      ClutterStageGLX   *stage_glx;
+      ClutterStageX11   *stage_x11;
 
-      impl = _clutter_stage_get_window (stage);
       g_assert (impl != NULL);
 
       CLUTTER_NOTE (MULTISTAGE, "Setting context for stage of type %s [%p]",
@@ -435,6 +504,7 @@ clutter_backend_glx_ensure_context (ClutterBackend *backend,
       stage_glx = CLUTTER_STAGE_GLX (impl);
       stage_x11 = CLUTTER_STAGE_X11 (impl);
       backend_glx = CLUTTER_BACKEND_GLX (backend);
+      backend_x11 = CLUTTER_BACKEND_X11 (backend);
 
       /* no GL context to set */
       if (backend_glx->gl_context == None)
@@ -447,26 +517,24 @@ clutter_backend_glx_ensure_context (ClutterBackend *backend,
        */
       if (stage_x11->xwin == None)
         {
-          ClutterBackendX11 *backend_x11;
-
-          backend_x11 = CLUTTER_BACKEND_X11 (backend);
           CLUTTER_NOTE (MULTISTAGE,
                         "Received a stale stage, clearing all context");
 
-          glXMakeCurrent (backend_x11->xdpy, None, NULL);
+          glXMakeContextCurrent (backend_x11->xdpy, None, None, NULL);
         }
       else
         {
           CLUTTER_NOTE (BACKEND,
-                        "MakeCurrent dpy: %p, window: 0x%x (%s), context: %p",
-                        stage_x11->xdpy,
+                        "MakeContextCurrent dpy: %p, window: 0x%x (%s), context: %p",
+                        backend_x11->xdpy,
                         (int) stage_x11->xwin,
                         stage_x11->is_foreign_xwin ? "foreign" : "native",
                         backend_glx->gl_context);
 
-          glXMakeCurrent (stage_x11->xdpy,
-                          stage_x11->xwin,
-                          backend_glx->gl_context);
+          glXMakeContextCurrent (backend_x11->xdpy,
+                                 stage_x11->xwin,
+                                 stage_x11->xwin,
+                                 backend_glx->gl_context);
         }
 
       if (clutter_x11_untrap_x_errors ())
@@ -539,6 +607,7 @@ static void
 clutter_backend_glx_redraw (ClutterBackend *backend,
                             ClutterStage   *stage)
 {
+  ClutterBackendX11 *backend_x11;
   ClutterStageGLX *stage_glx;
   ClutterStageX11 *stage_x11;
   ClutterStageWindow *impl;
@@ -552,6 +621,7 @@ clutter_backend_glx_redraw (ClutterBackend *backend,
 
   g_assert (CLUTTER_IS_STAGE_GLX (impl));
 
+  backend_x11 = CLUTTER_BACKEND_X11 (backend);
   stage_x11 = CLUTTER_STAGE_X11 (impl);
   stage_glx = CLUTTER_STAGE_GLX (impl);
 
@@ -567,96 +637,39 @@ clutter_backend_glx_redraw (ClutterBackend *backend,
 
       /* push on the screen */
       CLUTTER_NOTE (BACKEND, "glXSwapBuffers (display: %p, window: 0x%lx)",
-                    stage_x11->xdpy,
+                    backend_x11->xdpy,
                     (unsigned long) stage_x11->xwin);
-      glXSwapBuffers (stage_x11->xdpy, stage_x11->xwin);
-    }
-  else
-    {
-      /* offscreen */
-      glXWaitGL ();
-
-      CLUTTER_GLERR ();
+      glXSwapBuffers (backend_x11->xdpy, stage_x11->xwin);
     }
 }
 
-static ClutterActor *
+static ClutterStageWindow *
 clutter_backend_glx_create_stage (ClutterBackend  *backend,
                                   ClutterStage    *wrapper,
                                   GError         **error)
 {
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
-  ClutterStageX11   *stage_x11;
-  ClutterActor      *stage;
+  ClutterStageWindow *stage_window;
+  ClutterStageX11 *stage_x11;
 
   CLUTTER_NOTE (BACKEND, "Creating stage of type '%s'",
                 g_type_name (CLUTTER_STAGE_TYPE));
 
-  stage = g_object_new (CLUTTER_TYPE_STAGE_GLX, NULL);
+  stage_window = g_object_new (CLUTTER_TYPE_STAGE_GLX, NULL);
 
   /* copy backend data into the stage */
-  stage_x11 = CLUTTER_STAGE_X11 (stage);
-  stage_x11->xdpy = backend_x11->xdpy;
-  stage_x11->xwin_root = backend_x11->xwin_root;
-  stage_x11->xscreen = backend_x11->xscreen_num;
-  stage_x11->backend = backend_x11;
+  stage_x11 = CLUTTER_STAGE_X11 (stage_window);
   stage_x11->wrapper = wrapper;
 
   CLUTTER_NOTE (BACKEND,
                 "GLX stage created[%p] (dpy:%p, screen:%d, root:%u, wrap:%p)",
-                stage,
-                stage_x11->xdpy,
-                stage_x11->xscreen,
-                (unsigned int) stage_x11->xwin_root,
+                stage_window,
+                backend_x11->xdpy,
+                backend_x11->xscreen_num,
+                (unsigned int) backend_x11->xwin_root,
                 wrapper);
 
-  return stage;
-}
-
-static XVisualInfo *
-clutter_backend_glx_get_visual_info (ClutterBackendX11 *backend_x11,
-                                     gboolean           for_offscreen)
-{
-  XVisualInfo *xvisinfo;
-  int onscreen_gl_attributes[] = {
-    GLX_RGBA,
-    GLX_DOUBLEBUFFER,
-    GLX_RED_SIZE,     1,
-    GLX_GREEN_SIZE,   1,
-    GLX_BLUE_SIZE,    1,
-    GLX_ALPHA_SIZE,   1,
-    GLX_STENCIL_SIZE, 1,
-    GLX_DEPTH_SIZE,   1,
-    0
-  };
-  int offscreen_gl_attributes[] = {
-    GLX_RGBA,
-    GLX_USE_GL,
-    GLX_DEPTH_SIZE,   0,
-    GLX_ALPHA_SIZE,   0,
-    GLX_STENCIL_SIZE, 1,
-    GLX_RED_SIZE,     1,
-    GLX_GREEN_SIZE,   1,
-    GLX_BLUE_SIZE,    1,
-    0
-  };
-
-  if (backend_x11->xdpy == None || backend_x11->xscreen == None)
-    return NULL;
-
-  CLUTTER_NOTE (BACKEND,
-                "Retrieving GL visual (for %s use), dpy: %p, xscreen; %p (%d)",
-                for_offscreen ? "offscreen" : "onscreen",
-                backend_x11->xdpy,
-                backend_x11->xscreen,
-                backend_x11->xscreen_num);
-
-  xvisinfo = glXChooseVisual (backend_x11->xdpy,
-                              backend_x11->xscreen_num,
-                              for_offscreen ? offscreen_gl_attributes
-                                            : onscreen_gl_attributes);
-
-  return xvisinfo;
+  return stage_window;
 }
 
 static void
